@@ -2417,8 +2417,7 @@ int modem_init ( TFILE *mf, cap c, char *id,
   cmd ( mf, "E0", t ) ;
 
   if ( cmd ( mf, "I3", t ) == OK ) {
-    getresp ( "", model, CMDBUFSIZE-1 ) ;
-    strcat ( model, " " ) ;
+    getresp ( "", model, CMDBUFSIZE ) ;
   }
 
   /* if not already chosen, pick safest class; set it */
@@ -2446,8 +2445,7 @@ int modem_init ( TFILE *mf, cap c, char *id,
   if ( ! c1 ) {
     for ( p = modelq [ c20 ] ; ! err && *p ; p++ ) {
       if ( cmd ( mf, *p, t ) == OK ) {
-	getresp ( "", model, CMDBUFSIZE-1 ) ;
-	strcat ( model, " " ) ;
+	getresp ( "", model, CMDBUFSIZE ) ;
       }
     }
   
@@ -2461,7 +2459,7 @@ int modem_init ( TFILE *mf, cap c, char *id,
   if ( ! err ) {
     /* Translator: the first string is the model of modem and the
        second is the fax modem class in which it operates */
-    message = strdup2 ( "I ", gettext ( "using %sin class %s" ) ) ;
+    message = strdup2 ( "I ", gettext ( "using %s in class %s" ) ) ;
     if ( message ) {
       msg ( message, model, c1 ? "1" : c20 ? "2.0" : "2" ) ;
       free ( message ) ;
