@@ -36,6 +36,7 @@ COPYING distributed with the source files.
 #include "utils/shared_handle.h"
 
 #ifdef HAVE_OSTREAM
+#include <ios>
 #include <ostream>
 #endif
 
@@ -45,6 +46,10 @@ COPYING distributed with the source files.
 #include <strstream>
 #endif
 
+#ifdef HAVE_STREAM_IMBUE
+#include <locale>
+#endif
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -506,6 +511,11 @@ void EfaxController::make_fax_thread(void) {
     int partnumber = 1;
 #ifdef HAVE_STRINGSTREAM
     std::ostringstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << *filename_iter << '.' << std::setfill('0')
 	 << std::setw(3) << partnumber;
     int result = access(strm.str().c_str(), R_OK);
@@ -528,6 +538,11 @@ void EfaxController::make_fax_thread(void) {
     }
 #else
     std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << filename_iter->c_str() << '.' << std::setfill('0')
 	 << std::setw(3) << partnumber << std::ends;
     const char* test_name = strm.str();
@@ -543,6 +558,11 @@ void EfaxController::make_fax_thread(void) {
 
       partnumber++;
       std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+      strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
       strm << filename_iter->c_str() << '.' << std::setfill('0')
 	   << std::setw(3) << partnumber << std::ends;
       test_name = strm.str();
@@ -1112,10 +1132,20 @@ std::pair<std::string, std::string> EfaxController::save_sent_fax(void) {
 	int in_partnumber = 1;
 #ifdef HAVE_STRINGSTREAM
 	std::ostringstream in_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	in_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	in_suffix_strm << '.' << std::setfill('0')
 		       << std::setw(3) << in_partnumber;
 
 	std::ostringstream out_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	out_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	out_suffix_strm << '.' << std::setfill('0')
 			<< std::setw(3) << out_partnumber;
 
@@ -1125,11 +1155,21 @@ std::pair<std::string, std::string> EfaxController::save_sent_fax(void) {
 
 #else
 	std::ostrstream in_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	in_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	in_suffix_strm << '.' << std::setfill('0')
 		       << std::setw(3) << in_partnumber << std::ends;
 	const char* in_suffix = in_suffix_strm.str();
 
 	std::ostrstream out_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	out_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	out_suffix_strm << '.' << std::setfill('0')
 			<< std::setw(3) << out_partnumber << std::ends;
 	const char* out_suffix = out_suffix_strm.str();
@@ -1177,11 +1217,21 @@ std::pair<std::string, std::string> EfaxController::save_sent_fax(void) {
 
 #else
 	  std::ostrstream in_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	  in_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	  in_suffix_strm << '.' << std::setfill('0')
 			 << std::setw(3) << in_partnumber << std::ends;
 	  in_suffix = in_suffix_strm.str();
 
 	  std::ostrstream out_suffix_strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+	  out_suffix_strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
 	  out_suffix_strm << '.' << std::setfill('0')
 			  << std::setw(3) << out_partnumber << std::ends;
 	  out_suffix = out_suffix_strm.str();
@@ -1248,6 +1298,11 @@ void EfaxController::cleanup_fax_send_fail(void) {
     int partnumber = 1;
 #ifdef HAVE_STRINGSTREAM
     std::ostringstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << '.' << std::setfill('0') << std::setw(3) << partnumber;
 
     // make the suffixed file name
@@ -1262,6 +1317,11 @@ void EfaxController::cleanup_fax_send_fail(void) {
     }
 #else
     std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
     strm << '.' << std::setfill('0') << std::setw(3) << partnumber << std::ends;
     const char* suffix = strm.str();
 
@@ -1273,6 +1333,11 @@ void EfaxController::cleanup_fax_send_fail(void) {
       unlink(filename.c_str());
       partnumber++;
       std::ostrstream strm;
+
+#  ifdef HAVE_STREAM_IMBUE
+      strm.imbue(std::locale::classic());
+#  endif // HAVE_STREAM_IMBUE
+
       strm << '.' << std::setfill('0') << std::setw(3) << partnumber << std::ends;
       suffix = strm.str();
       filename = *filename_iter + suffix;
