@@ -14,10 +14,10 @@ COPYING distributed with the source files.
 #include <gtk/gtkprintjob.h>
 #endif
 
+#include <stdlib.h>  // include <stdlib.h> rather than <cstdlib> for mkstemp
 #include <iomanip>
 #include <memory>
 #include <cstring>
-#include <cstdlib>
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -957,7 +957,9 @@ void FaxListDialog::print_fax(void) {
 
     // create the print manager
 #if GTK_CHECK_VERSION(2,10,0)
-    if (prog_config.gtkprint) async_queue.push(PrintManager::create_manager(get_win()));
+    if (prog_config.gtkprint) async_queue.push(FilePrintManager::create_manager(get_win(),
+										gettext("efax-gtk: Print fax"),
+										prog_config.window_icon_h));
 #endif
 
     // now block off the signals for which we have set handlers so that the worker
@@ -1013,9 +1015,9 @@ void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
   // extract the string argument to a std::auto_ptr<> object
   const std::auto_ptr<std::string> fax_basename_a(arg_p);
 
-  // extract any PrintManager object
+  // extract any FilePrintManager object
 #if GTK_CHECK_VERSION(2,10,0)
-  IntrusivePtr<PrintManager> print_manager_i;
+  IntrusivePtr<FilePrintManager> print_manager_i;
   if (use_gtkprint) {
     try {
      async_queue.pop(print_manager_i);
@@ -1086,7 +1088,7 @@ void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
     while (::close(file_fd) == -1 && errno == EINTR);
 
     print_manager_i->set_filename(tempfile_h.get());
-    print_manager_i->print(); // this will pass ownership to the PrintManager object
+    print_manager_i->print(); // this will pass ownership to the FilePrintManager object
                               // which will control its own destiny now, via the GTK+
                               // print system
   }
