@@ -33,6 +33,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkcellrenderer.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeviewcolumn.h>
+#include <gtk/gtkversion.h>
 
 #include <gdk/gdkpixbuf.h>
 
@@ -43,6 +44,7 @@ COPYING distributed with the source files.
 #include "utils/tree_path_handle.h"
 #include "utils/shared_handle.h"
 #include "utils/utf8_utils.h"
+#include "utils/toolbar_append_widget.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -186,8 +188,8 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, view_button_p,
-			    gettext("View selected file"), "");
+  toolbar_append_widget(toolbar_p, view_button_p,
+			gettext("View selected file"), "");
   gtk_widget_set_sensitive(view_button_p, false);
 
   { // scope block for the GobjHandle
@@ -196,8 +198,8 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(remove_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, remove_button_p,
-			    gettext("Remove selected file from list"), "");
+  toolbar_append_widget(toolbar_p, remove_button_p,
+			gettext("Remove selected file from list"), "");
   gtk_widget_set_sensitive(remove_button_p, false);
 
   GtkWidget* add_hbox_p = gtk_hbox_new(false, 0);
@@ -210,8 +212,8 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     gtk_box_pack_start(GTK_BOX(add_hbox_p), label_p, false, false, 4);
     gtk_container_add(GTK_CONTAINER(add_button_p), add_hbox_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, add_button_p,
-			    gettext("Add files to the file list for faxing"), "");
+  toolbar_append_widget(toolbar_p, add_button_p,
+			gettext("Add files to the file list for faxing"), "");
 
   // bring up the icon size registered in MainWindow::MainWindow()
   GtkIconSize efax_gtk_button_size = gtk_icon_size_from_name("EFAX_GTK_BUTTON_SIZE");
@@ -295,15 +297,19 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_default_size(get_win(), standard_size * 15, standard_size * 12);
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
+#endif
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 
   gtk_button_set_relief(GTK_BUTTON(add_button_p), GTK_RELIEF_NORMAL);
   gtk_widget_set_sensitive(add_button_p, true);
   
-  /* Not needed for GtkButton
+  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  /*
   gtk_widget_set_size_request(GTK_WIDGET(toolbar_p), 
 			      add_button_p->allocation.width + view_button_p->allocation.width
-			      + remove_button_p.allocation.width + 12,
+			      + remove_button_p->allocation.width + 12,
 			      add_button_p->allocation.height);
   */
 }
