@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2007 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -19,10 +19,12 @@ COPYING distributed with the source files.
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <cstdlib>
 #include <fstream>
@@ -45,6 +47,12 @@ COPYING distributed with the source files.
 #include <istream>
 #endif
 
+#ifdef HAVE_STRINGSTREAM
+#include <sstream>
+#else
+#include <strstream>
+#endif
+
 #ifdef HAVE_STREAM_IMBUE
 #include <locale>
 #endif
@@ -88,12 +96,17 @@ COPYING distributed with the source files.
 #include "utils/fdstream.h"
 #include "utils/pipes.h"
 #include "utils/notifier.h"
+#include "utils/gerror_handle.h"
 
 #ifdef ENABLE_NLS
 #include <locale.h>
 #include <libintl.h>
 #endif
 
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#endif
+
 #define LOCKFILE_NAME "/.efax-gtk-pid"
 
 // the headers on one of my machines does not properly include the declaration
@@ -121,6 +134,7 @@ bool is_arg(const char*, int, char*[]);
 extern "C" void atexit_cleanup(void);
 void clean_up_receive_dir(void);
 bool is_ascii(const std::string&);
+int is_in_process_table(pid_t);
 
 
 int main(int argc, char* argv[]) {
@@ -394,6 +408,7 @@ std::string configure_prog(bool reread) {
       prog_config.page_size = "";
       prog_config.page_dim = "";
       prog_config.resolution = "";
+      prog_config.dial_prefix = "";
       prog_config.print_cmd = "";
       prog_config.print_shrink = "";
       prog_config.ps_view_cmd = "";
@@ -457,6 +472,9 @@ std::string configure_prog(bool reread) {
 	// look for "RES:"
 	else if (get_prog_parm("RES:", file_read, prog_config.resolution));
 	
+	// look for "DIAL_PREFIX:"
+	else if (get_prog_parm("DIAL_PREFIX:", file_read, prog_config.dial_prefix));
+	
 	// look for "RINGS:"
 	else if (get_prog_parm("RINGS:", file_read, rings));
 	
@@ -952,7 +970,14 @@ void get_window_icon(void) {
       prog_config.window_icon_h =
 	GobjHandle<GdkPixbuf>(gdk_pixbuf_new_from_file(icon_path_p, &error_p));
       if (prog_config.window_icon_h.get()) have_icon = true;
-      else write_error("Pixbuf error in get_window_icon()\n");
+      else {
+	write_error("Pixbuf error in get_window_icon()\n");
+	if (error_p) {
+	  GerrorScopedHandle handle_h(error_p);
+	  write_error(error_p->message);
+	  write_error("\n");
+	}
+      }
     }
   }
 #endif
@@ -1034,9 +1059,12 @@ bool is_program_running(void) {
 	existing_lock.imbue(std::locale::classic());
 #endif // HAVE_STREAM_IMBUE
 
+	// test with kill() as well as with is_in_process_table()
+	// in case SUS 'ps' arguments are not available
 	pid_t existing_lock_pid;
 	existing_lock >> existing_lock_pid;
-	if (existing_lock && (kill(existing_lock_pid, 0)) < 0) { // stale lock
+	if (!is_in_process_table(existing_lock_pid)
+	    || (existing_lock && kill(existing_lock_pid, 0) < 0)) { // stale lock
 	  unlink(lockfile.c_str());    // delete stale lock file
 	  new_lock_fd = open(lockfile.c_str(), O_WRONLY | O_CREAT | O_EXCL,
 			     S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
@@ -1096,6 +1124,84 @@ bool is_ascii(const std::string& text) {
   return false;
 }
 
+// returns 1 if efax-gtk is definitely in the process table for the PID in existing_lock_pid
+// returns 0 if efax-gtk is definitely not in process table for the PID in existing_lock_pid
+// returns -1 if we cannot tell because 'ps' fails (say because SUS ps arguments are not available)
+int is_in_process_table(pid_t existing_lock_pid) {
+
+  // call ourselves recursively to see whether we can find out own process id in the process table
+  static bool checked = false;
+  if (!checked) {
+    checked = true;
+    if (is_in_process_table(getpid()) != 1) {
+      const std::string message("'ps' command is not POSIX conforming or is not in PATH\n");
+      write(2, message.data(), message.size());
+      return -1;
+    }
+  }
+
+  PipeFifo fork_pipe(PipeFifo::block);
+  // now fork to create the process which send the output of 'ps' to stdout
+  pid_t pid = fork();
+
+  if (pid == -1) {
+    const std::string message("Fork error - exiting\n");
+    write(2, message.data(), message.size());
+    std::exit(FORK_ERROR);
+  }
+  
+  if (!pid) { // child process which will send the output of 'ps' to stdout
+  
+    // this is the child process to write to stdout
+    fork_pipe.connect_to_stdout();
+
+    // the program is not multi-threaded at this point so there is no problem
+    // with using non async-signal-safe functions before we exec()
+#ifdef HAVE_STRINGSTREAM
+    std::ostringstream strm;
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif
+    strm << "-p" << existing_lock_pid;
+    execlp("ps", "ps", strm.str().c_str(), static_cast<char*>(0));
+#else
+    std::ostrstream strm;
+#  ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#  endif
+    strm << "-p" << existing_lock_pid << std::ends;
+    execlp("ps", "ps", strm.str(), static_cast<char*>(0));
+#endif
+
+    // if we reached this point, then the execlp() call must have failed
+    _exit(EXIT_SUCCESS - 1);
+  }
+
+  // read from the pipe
+  fork_pipe.make_readonly();
+  ssize_t result;
+  char buffer[PIPE_BUF];
+  std::string output;
+  while ((result = fork_pipe.read(buffer, PIPE_BUF)) > 0) {
+    output.append(buffer, result);
+  }
+
+  write(1, output.data(), output.size());
+  // we can safely use wait() here because we are setting the program up
+  // and nothing else is calling wait() or waitpid() and we do not need
+  // to worry about EINTR because there are no signals at this stage
+  int status;
+  if (output.empty() || wait(&status) == -1) return -1;
+  int exit_code = EXIT_SUCCESS - 1;
+  if (WIFEXITED(status)) exit_code = WEXITSTATUS(status);
+  if (exit_code != EXIT_SUCCESS) return -1;
+
+  if (output.find("efax-gtk") != std::string::npos) {
+    return 1;
+  }
+  return 0;
+}
+
 void atexit_cleanup(void) {
 
   // this does any essential clean up, which is always required to make the program
