@@ -94,6 +94,15 @@ public:
   using SettingsMessagesBase::get_message;
 };
 
+class SendMessages: private SettingsMessagesBase {
+public:
+  enum {res = 0, header, dial_prefix};
+  SendMessages(void);
+
+  using SettingsMessagesBase::get_caption;
+  using SettingsMessagesBase::get_message;
+};
+
 class LoggingMessages: private SettingsMessagesBase {
 public:
   enum {logfile = 0};
@@ -105,7 +114,7 @@ public:
 
 class PageMessages: private SettingsMessagesBase {
 public:
-  enum {page = 0, res, header};
+  enum {page = 0};
   PageMessages(void);
 
   using SettingsMessagesBase::get_caption;
