@@ -31,6 +31,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkcellrenderer.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeviewcolumn.h>
+#include <gtk/gtkversion.h>
 
 #include <gdk/gdkpixbuf.h>
 
@@ -38,6 +39,7 @@ COPYING distributed with the source files.
 #include "dialogs.h"
 #include "socket_list_icons.h"
 #include "utils/utf8_utils.h"
+#include "utils/toolbar_append_widget.h"
 
 #ifdef HAVE_OSTREAM
 #include <ios>
@@ -182,8 +184,8 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, view_button_p,
-			    gettext("View selected file"), "");
+  toolbar_append_widget(toolbar_p, view_button_p,
+			gettext("View selected file"), "");
   gtk_widget_set_sensitive(view_button_p, false);
 
   { // scope block for the GobjHandle
@@ -192,8 +194,8 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(remove_button_p), image_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, remove_button_p,
-			    gettext("Remove selected file from list"), "");
+  toolbar_append_widget(toolbar_p, remove_button_p,
+			gettext("Remove selected file from list"), "");
   gtk_widget_set_sensitive(remove_button_p, false);
 
   GtkWidget* send_hbox_p = gtk_hbox_new(false, 0);
@@ -206,8 +208,8 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     gtk_box_pack_start(GTK_BOX(send_hbox_p), label_p, false, false, 4);
     gtk_container_add(GTK_CONTAINER(send_button_p), send_hbox_p);
   }
-  gtk_toolbar_append_widget(toolbar_p, send_button_p,
-			    gettext("Choose the selected fax for sending"), "");
+  toolbar_append_widget(toolbar_p, send_button_p,
+			gettext("Choose the selected fax for sending"), "");
   gtk_widget_set_sensitive(send_button_p, false);
 
   gtk_table_attach(table_p, GTK_WIDGET(socket_list_scrolled_window_p),
@@ -243,12 +245,16 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   gtk_container_add(GTK_CONTAINER(get_win()), socket_list_box_p);
   //gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER);
   gtk_window_set_default_size(get_win(), standard_size * 15, standard_size * 12);
+#if GTK_CHECK_VERSION(2,4,0)
+  gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
+#endif
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 
-  /* Not needed for GtkButton
+  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  /*
   gtk_widget_set_size_request(GTK_WIDGET(toolbar_p), 
 			      send_button_p->allocation.width + view_button_p->allocation.width
-			      + remove_button_p.allocation.width + 12,
+			      + remove_button_p->allocation.width + 12,
 			      send_button_p->allocation.height);
   */
 }
