@@ -6,7 +6,7 @@ COPYING distributed with the source files.
 
 */
 
-#include <cstdlib>
+#include <stdlib.h>  // include <stdlib.h> rather than <cstdlib> for mkstemp
 #include <fstream>
 #include <vector>
 #include <algorithm>
