@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 and 2006 Chris Vine
+/* Copyright (C) 2005 and 2007 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -83,6 +83,14 @@ modifications of the source code in this file).
    conversion operator (which returns the underlying pointer), or by
    explicitly calling the get() method to obtain the underlying
    pointer.
+
+   As of glib-2.8, g_object_ref() and g_object_unref() are thread
+   safe, so with glib-2.8 or greater you can have different GobjHandle
+   instances in different threads referencing the same GObject object.
+   Of course, if you do so, this does not affect the need (or
+   otherwise) in the particular use in question to lock anything other
+   than the reference count - say when accessing the referenced object
+   itself in different threads.
 */
 
 #include <glib-object.h>
