@@ -42,8 +42,8 @@ modifications of the source code in this file).
   reference count.  It requires that the referenced object has two
   functions called ref() and unref(), which increment and decrement
   the reference count respectively.  The IntrusiveCounter or
-  IntrusiveLockedCounter class can be inherited from to do this, but
-  they do not have to be used.  The IntrusiveLockedCounter class is
+  IntrusiveLockCounter class can be inherited from to do this, but
+  they do not have to be used.  The IntrusiveLockCounter class is
   the same as the IntrusiveCounter class, except that it locks the
   reference count when it is incremented or decremented in order that
   IntrusivePtr objects in different threads can access the same
@@ -54,13 +54,13 @@ modifications of the source code in this file).
   reference count, so it should only normally be passed a newly
   created object which begins with a reference count of 1, assuming
   that the object will arrange to delete itself once the reference
-  count falls to 0.  The IntrusiveCounter and IntrusiveLockedCounter
+  count falls to 0.  The IntrusiveCounter and IntrusiveLockCounter
   classes behave in this way.
 */
 
 // define this if you want to use GLIB atomic functions/memory barriers
 // instead of a (slower) mutex to lock the reference count in the
-// IntrusiveLockedCounter class - to do this at least glib version 2.4
+// IntrusiveLockCounter class - to do this at least glib version 2.4
 // is required
 #define INTRUSIVE_LOCK_COUNTER_USE_GLIB_ATOMIC_OPS 1
 
@@ -158,6 +158,16 @@ public:
 class IntrusiveCounter {
   unsigned int count;
 
+  // we should not be able to copy objects of this class
+  // - objects, if constructed on the heap, should be passed
+  // via an IntrusivePtr object.  An object of a derived class
+  // might still copy itself via its copy constructor and take
+  // along a new base IntrusiveCounter object constructed via
+  // the default constructor, and thus have a correct reference
+  // count of 1, but derived classes should not try to provide
+  // their own assignment operators.
+  IntrusiveCounter(const IntrusiveCounter&);
+  IntrusiveCounter& operator=(const IntrusiveCounter&);
 public:
   // this method does not throw
   void ref(void) {++count;}
@@ -181,6 +191,16 @@ class IntrusiveLockCounter {
   gint count;
 #endif
 
+  // we should not be able to copy objects of this class
+  // - objects, if constructed on the heap, should be passed
+  // via an IntrusivePtr object.  An object of a derived class
+  // might still copy itself via its copy constructor and take
+  // along a new base IntrusiveLockCounter object constructed via
+  // the default constructor, and thus have a correct reference
+  // count of 1, but derived classes should not try to provide
+  // their own assignment operators.
+  IntrusiveLockCounter(const IntrusiveLockCounter&);
+  IntrusiveLockCounter& operator=(const IntrusiveLockCounter&);
 public:
   // this method does not throw
   void ref(void) {
