@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 and 2006 Chris Vine
+/* Copyright (C) 2005 to 2007 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -28,7 +28,7 @@ struct WatchSource {
   GSource source;
   GPollFD poll_fd;
   GIOCondition io_condition;
-  sigc::signal0<bool>* dispatch_sig_p;
+  sigc::slot<bool>* dispatch_slot_p;
 };
 
 extern "C" {
@@ -71,7 +71,7 @@ gboolean io_watch_dispatch_func(GSource* source_p, GSourceFunc, void*) {
 
   // we are not interested in the GSourceFunc argument here as we have never
   // called g_source_set_callback()
-  return watch_source_p->dispatch_sig_p->emit();
+  return (*watch_source_p->dispatch_slot_p)();
 }
 
 void io_watch_finalize_func(GSource* source_p) {
@@ -82,8 +82,8 @@ void io_watch_finalize_func(GSource* source_p) {
   // both are PODSs
   WatchSource* watch_source_p = reinterpret_cast<WatchSource*>(source_p);
   
-  delete watch_source_p->dispatch_sig_p;
-  watch_source_p->dispatch_sig_p = 0;
+  delete watch_source_p->dispatch_slot_p;
+  watch_source_p->dispatch_slot_p = 0;
 }
 
 GSourceFuncs io_watch_source_funcs = {
@@ -111,10 +111,7 @@ guint start_iowatch(int fd, const sigc::slot<bool>& callback,
   watch_source_p->poll_fd.events = io_condition;
   watch_source_p->poll_fd.revents = 0;
   watch_source_p->io_condition = io_condition;
-  watch_source_p->dispatch_sig_p = new sigc::signal0<bool>;
-
-  // connect up the callback
-  watch_source_p->dispatch_sig_p->connect(callback);
+  watch_source_p->dispatch_slot_p = new sigc::slot<bool>(callback);
 
   // connect the source object to its polling object  
   g_source_add_poll(source_p, &watch_source_p->poll_fd);
