@@ -38,13 +38,14 @@ License as follows:
 
    If the signalling thread is the same thread as that in which the
    slot connected to it will execute (which is the thread in which the
-   object was constructed), executing the slot via the pipe would risk
-   a deadlock - if the pipe fills up, the thread would block on write
-   and never be able to read from the pipe to empty it.  Accordingly,
-   if the object is invoked by the same thread as that in which the
-   slot will execute, this is detected and the slot will be invoked
-   directly, rather than via the pipe.  Therefore, actions so invoked
-   may be out of order with those invoked by the other threads.
+   the main Glib program event loop executes), executing the slot via
+   the pipe would risk a deadlock - if the pipe fills up, the thread
+   would block on write and never be able to read from the pipe to
+   empty it.  Accordingly, if the object is invoked by the same thread
+   as that in which the slot will execute, this is detected and the
+   slot will be invoked directly, rather than via the pipe.
+   Therefore, actions so invoked may be out of order with those
+   invoked by the other threads.
 
    The main use of Notifier objects is for a worker thread to signal
    an event to the main thread in which GTK+ is executing, which
@@ -63,11 +64,13 @@ License as follows:
    occur automatically on that first construction.  That means that if
    the first Notifier object is not constructed in the main (event
    loop) thread of the program, then Notifier::init() must be called
-   explicitly before that first object is constructed.  It is a good
-   idea also that Notifier::init() should have been called (or the
-   first Notifier object constructed) before the main program thread
-   creates any new threads.  Then the state of Notifier::initialised
-   will automatically be visible between threads.
+   explicitly before that first object is constructed.  Before
+   Notifier::init() is called (or the first Notifier object created)
+   g_thread_init(0) should have been called.  It is a good idea also
+   that Notifier::init() should have been called (or the first
+   Notifier object constructed) before the main program thread creates
+   any new threads.  Then the state of Notifier::initialised will
+   automatically be visible between threads.
 
    The Notifier::read_pipe_slot() method checks for a case where
    between the signalling thread invoking a Notifier object and the
@@ -82,6 +85,18 @@ License as follows:
    synchronisation to secure the lifetime of the Notifier object until
    after the slot has been called.
 
+   The Notifier::connect() method connects a slot to the sigc::signal
+   member of the Notifier object.  If the slot function is a member
+   function of a class which derives from sigc::trackable, it will
+   invoke sigc methods which are not necessarily thread safe, so any
+   connections to such a slot should be done in only one thread if
+   additional synchronisation is to be avoided.  (A good rule is to
+   invoke Notifier::connect() only in the thread in which the main
+   Glib program event loop executes, which is the same thread as the
+   one in which the slot itself will execute, so that automatic
+   visibility and synchronisation of relevant sigc variables is
+   assured without further synchronisation.)
+
    For a program with two GMainContext program event loops (not a
    usual case), it would be possible for a Notifier-like object to be
    initialised in the non-default GMainContext thread, and execute in
@@ -106,6 +121,9 @@ License as follows:
    filling up before being emptied by the Notifier::read_pipe_slot()
    callback function executing in the main program and so blocking in
    the handler, thus deadlocking the program.
+
+   To pass variable data to the slot function executed by the Notifier
+   object, the AsyncQueue class can be employed.
 */
 
 
