@@ -26,16 +26,18 @@ License as follows:
 #include "thread.h"
 #include "sem_sync.h"
 
+namespace Thread {
 struct ThreadArgs {
   const sigc::slot<void>* slot_p;
   SemSync* sem_p;
 };
+} // namespace Thread
 
 namespace {
-void* thread_func(void* arg) {
+void* ThreadCB::thread_func(void* arg) {
 
   // this is executing in the new thread
-  ThreadArgs* thread_args_p = static_cast<ThreadArgs*>(arg);
+  Thread::ThreadArgs* thread_args_p = static_cast<Thread::ThreadArgs*>(arg);
   sigc::slot<void> callback = *(thread_args_p->slot_p);
   thread_args_p->sem_p->post();
   // at this point the pointers in pthread_args will cease to be valid
@@ -74,7 +76,7 @@ std::auto_ptr<Thread> Thread::start(const sigc::slot<void>& cb, bool joinable) {
   thread_args.sem_p = &sem;
 
   pthread_t thread;
-  if (!pthread_create(&thread, &attr, ::thread_func, &thread_args)) {
+  if (!pthread_create(&thread, &attr, ThreadCB::thread_func, &thread_args)) {
     instance_p->thread = thread;
     sem.wait();
   }
